import { useState } from "react";
import { useNavigate } from "react-router-dom";
import Box from "@mui/material/Box";
import Button from "@mui/material/Button";
import FormControl from "@mui/material/FormControl";
import { useCreateServer } from "@mcl/queries";

// Core Options
import NameOption from "@mcl/components/server-options/NameOption.jsx";
import HostOption from "@mcl/components/server-options/HostOption.jsx";
import VersionOption from "@mcl/components/server-options/VersionOption.jsx";
import ServerTypeOption, {
  serverTypeOptions,
} from "@mcl/components/server-options/ServerTypeOption.jsx";
import CpuOption, {
  cpuOptions,
} from "@mcl/components/server-options/CpuOption.jsx";
import MemoryOption, {
  memoryOptions,
} from "@mcl/components/server-options/MemoryOption.jsx";

const defaultServer = {
  version: "latest",
  serverType: serverTypeOptions[0],
  cpu: cpuOptions[0],
  memory: memoryOptions[2], // 1.5GB
};

export default function CreateCoreOptions() {
  const [spec, setSpec] = useState(defaultServer);
  const nav = useNavigate();
  const createServer = useCreateServer(spec);

  const updateSpec = (attr, val) => {
    const s = { ...spec };
    s[attr] = val;
    setSpec(s);
  };

  const coreUpdate = (attr) => (e) => updateSpec(attr, e.target.value);

  async function upsertSpec() {
    if (validateSpec() !== "validated") return;
    createServer(spec)
      .then(() => nav("/"))
      .catch(alert);
  }

  function validateSpec() {
    console.log("TODO CREATE VALIDATION");
    if (!spec.host) return alertValidationError("Host cannot be blank");
    if (!spec.name) return alertValidationError("Name not included");
    if (!spec.version) return alertValidationError("Version cannot be blank");
    return "validated";
  }

  function alertValidationError(reason) {
    alert(`Could not validate spec because: ${reason}`);
  }

  return (
    <Box
      className="create-options"
      sx={{ width: "100%", maxWidth: "600px", margin: "auto" }}
    >
      <FormControl fullWidth sx={{ mt: "2rem", display: "flex", gap: ".5rem" }}>
        <NameOption onChange={coreUpdate("name")} />
        <HostOption onChange={coreUpdate("host")} />
        <VersionOption value={spec.version} onChange={coreUpdate("version")} />
        <ServerTypeOption
          value={spec.serverType}
          onChange={coreUpdate("serverType")}
        />
        <CpuOption value={spec.cpu} onChange={coreUpdate("cpu")} />
        <MemoryOption value={spec.memory} onChange={coreUpdate("memory")} />
        <Button onClick={upsertSpec} variant="contained">
          Create
        </Button>
      </FormControl>
    </Box>
  );
}
